/*!
 * @file 
 * @brief LTEIoT6 Click example
 * 
 * # Description
 * This example reads and processes data from LTE IoT 6 clicks.
 *
 * The demo application is composed of two sections :
 * 
 * ## Application Init 
 * Initializes driver and wake-up module.
 * 
 * ## Application Task  
 * Reads the received data and parses it.
 * 
 * ## Additional Function
 * - static void lteiot6_clear_app_buf ( void )
 * - static void lteiot6_error_check( err_t error_flag )
 * - static void lteiot6_log_app_buf ( void )
 * - static void lteiot6_check_connection( void )
 * - static err_t lteiot6_rsp_check ( void )
 * - static err_t lteiot6_process ( void )
 * 
 * #Note: 
 * In order for the example to work, 
   user needs to set the phone number and sim apn to which he wants to send an SMS
 * Enter valid data for the following macros: SIM_APN and PHONE_NUMBER_TO_MESSAGE.
 * E.g. 
    SIM_APN "vip.mobile"
    PHONE_NUMBER_TO_MESSAGE "999999999"
 * 
 * @author Luka Filipovic
 *
 */
// ------------------------------------------------------------------- INCLUDES

#include "board.h"
#include "log.h"
#include "lteiot6.h"

#define APP_OK                              0
#define APP_ERROR_DRIVER                    -1
#define APP_ERROR_OVERFLOW                  -2
#define APP_ERROR_TIMEOUT                   -3

#define RSP_OK                              "OK"
#define RSP_ERROR                           "ERROR"

#define SIM_APN                             ""  // Set valid SIM APN
#define PHONE_NUMBER_TO_MESSAGE             ""  // Set Phone number to message
#define MESSAGE_CONTENT                     "LTE IoT 6 Click"   // Messege content 

#define PROCESS_BUFFER_SIZE                 500

#define WAIT_FOR_CONNECTION                 0
#define CONNECTED_TO_NETWORK                1

static lteiot6_t lteiot6;
static log_t logger;

static char app_buf[ PROCESS_BUFFER_SIZE ]  = { 0 };
static int32_t app_buf_len                  = 0;
static int32_t app_buf_cnt                  = 0;

static uint8_t app_connection_status        = WAIT_FOR_CONNECTION;

static err_t app_error_flag;

/**
 * @brief LTE IoT 6 clearing application buffer.
 * @details This function clears memory of application buffer and reset it's length and counter.
 * @note None.
 */
static void lteiot6_clear_app_buf ( void );

/**
 * @brief LTE IoT 6 data reading function.
 * @details This function reads data from device and concats data to application buffer.
 * 
 * @return @li @c  0 - Read some data.
 *         @li @c -1 - Nothing is read.
 *         @li @c -2 - Application buffer overflow.
 *
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t lteiot6_process ( void );

/**
 * @brief LTE IoT 6 check for errors.
 * @details This function checks for different types of errors and logs them on UART.
 * @note None.
 */
static void lteiot6_error_check( err_t error_flag );

/**
 * @brief LTE IoT 6 logs application buffer.
 * @details This function logs data from application buffer.
 * @note None.
 */
static void lteiot6_log_app_buf ( void );

/**
 * @brief LTE IoT 6 response check.
 * @details This function checks for response and returns the status of response.
 * 
 * @return application status.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t lteiot6_rsp_check ( void );

/**
 * @brief LTE IoT 6 chek connection.
 * @details This function checks connection to the network and 
 *          logs that status to UART.
 * 
 * @note None.
 */
static void lteiot6_check_connection( void );


void application_init ( void )
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    lteiot6_cfg_t lteiot6_cfg;  /**< Click config object. */

    // Logger initialization.
    LOG_MAP_USB_UART( log_cfg );
    log_cfg.level = LOG_LEVEL_DEBUG;
    log_cfg.baud = 115200;
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );
    Delay_ms( 1000 );
    
    // Click initialization.
    lteiot6_cfg_setup( &lteiot6_cfg );
    LTEIOT6_MAP_MIKROBUS( lteiot6_cfg, MIKROBUS_1 );
    err_t init_flag  = lteiot6_init( &lteiot6, &lteiot6_cfg );
    if ( init_flag == UART_ERROR )
    {
        log_error( &logger, " Application Init Error. " );
        log_info( &logger, " Please, run program again... " );

        for ( ; ; );
    }
    
    log_info( &logger, " Power up device... " );
    lteiot6_reset( &lteiot6 );
    lteiot6_power_on( &lteiot6 );
    
    lteiot6_send_cmd( &lteiot6, LTEIOT6_CMD_AT );
    lteiot6_process(  );
    lteiot6_clear_app_buf(  );
    
    //AT
    lteiot6_send_cmd( &lteiot6, LTEIOT6_CMD_AT );
    app_error_flag = lteiot6_rsp_check();
    lteiot6_error_check( app_error_flag );
    Delay_ms( 1000 );
    
    //ATI
    lteiot6_send_cmd( &lteiot6, LTEIOT6_CMD_ATI );
    app_error_flag = lteiot6_rsp_check();
    lteiot6_error_check( app_error_flag );
    Delay_ms( 1000 );
    
    //CFUN
    lteiot6_send_cmd_with_parameter( &lteiot6, LTEIOT6_CMD_CFUN, "1" );
    app_error_flag = lteiot6_rsp_check();
    lteiot6_error_check( app_error_flag );
    Delay_ms( 1000 );
    
    //CEREG
    lteiot6_send_cmd_with_parameter( &lteiot6, LTEIOT6_CMD_CREG, "2" );
    app_error_flag = lteiot6_rsp_check();
    lteiot6_error_check( app_error_flag );
    Delay_ms( 1000 );
    
    //CGDCONT
    lteiot6_set_sim_apn( &lteiot6, SIM_APN );
    app_error_flag = lteiot6_rsp_check();
    lteiot6_error_check( app_error_flag );
    Delay_ms( 1000 );
    
    //COPS
    lteiot6_send_cmd_with_parameter( &lteiot6, LTEIOT6_CMD_COPS, "0" );
    app_error_flag = lteiot6_rsp_check();
    lteiot6_error_check( app_error_flag );
    Delay_ms( 1000 );
    
    //CIMI
    lteiot6_send_cmd( &lteiot6, LTEIOT6_CMD_CIMI );
    app_error_flag = lteiot6_rsp_check();
    lteiot6_error_check( app_error_flag );
    Delay_ms( 1000 );
    
    app_buf_len = 0;
    app_buf_cnt = 0;
    app_connection_status = WAIT_FOR_CONNECTION;
    log_info( &logger, " Application Task " );
    Delay_ms( 2000 );
}

void application_task ( void )
{
    
    if ( app_connection_status == WAIT_FOR_CONNECTION )
    {
        lteiot6_send_cmd_check( &lteiot6, LTEIOT6_CMD_CGATT );
        app_error_flag = lteiot6_rsp_check();
        lteiot6_error_check( app_error_flag );
        Delay_ms( 2000 );
        
        lteiot6_send_cmd_check( &lteiot6, LTEIOT6_CMD_CREG );
        app_error_flag = lteiot6_rsp_check();
        lteiot6_error_check( app_error_flag );
        Delay_ms( 2000 );
        
        lteiot6_send_cmd( &lteiot6, LTEIOT6_CMD_CSQ );
        app_error_flag = lteiot6_rsp_check();
        lteiot6_error_check( app_error_flag );
        Delay_ms( 2000 );
    }
    else
    {
        //Setting SMS mode
        lteiot6_send_cmd_with_parameter( &lteiot6, "AT+CMGF", "1" );
        app_error_flag = lteiot6_rsp_check();
        lteiot6_error_check( app_error_flag );
        Delay_ms( 3000 );
        
        for( ; ; )
        {   
            log_printf( &logger, "> Sending message to phone number...\r\n" );
            lteiot6_send_text_message( &lteiot6, PHONE_NUMBER_TO_MESSAGE, MESSAGE_CONTENT );
            app_error_flag = lteiot6_rsp_check();
            lteiot6_error_check( app_error_flag );
            Delay_ms( 10000 );
        }
    }
}

void main ( void )
{
    Delay_100ms();
    application_init( );

    for ( ; ; )
    {
        application_task( );
    }
}

static void lteiot6_clear_app_buf ( void )
{
    memset( app_buf, 0, app_buf_len );
    app_buf_len = 0;
    app_buf_cnt = 0;
}

static err_t lteiot6_process ( void )
{
    err_t return_flag = APP_ERROR_DRIVER;
    int32_t rx_size;
    char rx_buff[ PROCESS_BUFFER_SIZE ] = { 0 };
    
    rx_size = lteiot6_generic_read( &lteiot6, rx_buff, PROCESS_BUFFER_SIZE );

    if ( rx_size > 0 )
    { 
        int32_t buf_cnt = 0;
        return_flag = APP_OK;

        if ( app_buf_len + rx_size >= PROCESS_BUFFER_SIZE )
        {
            lteiot6_clear_app_buf(  );
            return_flag = APP_ERROR_OVERFLOW;
        }
        else
        {
            buf_cnt = app_buf_len;
            app_buf_len += rx_size;
        }

        for ( int32_t rx_cnt = 0; rx_cnt < rx_size; rx_cnt++ )
        {
            if ( rx_buff[ rx_cnt ] != 0 ) 
            {
                app_buf[ ( buf_cnt + rx_cnt ) ] = rx_buff[ rx_cnt ];
            }
            else
            {
                app_buf_len--;
            }
        }
    } 

    return return_flag;
}

static err_t lteiot6_rsp_check ( void )
{
    uint16_t timeout_cnt = 0;
    uint16_t timeout = 10000;
    
    err_t error_flag = lteiot6_process(  );
    if ( ( error_flag != 0 ) && ( error_flag != -1 ) )
    {
        return error_flag;
    }
    
    while ( ( strstr( app_buf, RSP_OK ) == 0 ) && ( strstr( app_buf, RSP_ERROR ) == 0 ) )
    {
        error_flag = lteiot6_process(  );
        if ( ( error_flag != 0 ) && ( error_flag != -1 ) )
        {
            return error_flag;
        }
        
        timeout_cnt++;
        if ( timeout_cnt > timeout )
        {
            lteiot6_clear_app_buf(  );
            return APP_ERROR_TIMEOUT;
        }
        
        Delay_ms( 1 );
    }
    
    lteiot6_check_connection();
    
    lteiot6_log_app_buf();
    
    return APP_OK;
}

static void lteiot6_error_check( err_t error_flag )
{
    if ( ( error_flag != 0 ) && ( error_flag != -1 ) )
    {
        switch ( error_flag )
        {
            case -2:
                log_error( &logger, " Overflow!" );
                break;
            case -3:
                log_error( &logger, " Timeout!" );
                break;
            default:
                break;
        }
    }
}

static void lteiot6_log_app_buf ( void )
{
    for ( int32_t buf_cnt = 0; buf_cnt < app_buf_len; buf_cnt++ )
    {
        log_printf( &logger, "%c", app_buf[ buf_cnt ] );
    }
    log_printf( &logger, "\r\n" );
    lteiot6_clear_app_buf(  );
}

static void lteiot6_check_connection( void )
{
    #define CHECK_CONNECTION "AT+CGATT?"
    #define CONNECTED "+CGATT: 1"
    #define NOT_CONNECTED "+CGATT: 0"
    
    if ( strstr( app_buf, CHECK_CONNECTION ) != 0 )
    {
        if ( strstr( app_buf, CONNECTED ) != 0 )
        {
            log_info( &logger, "CONNECTED TO NETWORK" );
            app_connection_status = CONNECTED_TO_NETWORK;
        }
        else if ( strstr( app_buf, NOT_CONNECTED ) != 0 )
        {
            log_info( &logger, "NOT CONNECTED TO NETWORK" );
        }
    }
}

// ------------------------------------------------------------------------ END
